/*  Copyright @ 2018 gamefreak955

	All rights reserved.
	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:

	1. Redistributions of source code must retain the above copyright notice, this
	list of conditions and the following disclaimer.
	2. Redistributions in binary form must reproduce the above copyright notice,
	this list of conditions and the following disclaimer in the documentation
	and/or other materials provided with the distribution.
	3. Neither the name of this organization nor the names of its contributors may
	be used to endorse or promote products derived from this software without
	specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
	ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
	ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

	The views and conclusions contained in the software and documentation are those
	of the authors and should not be interpreted as representing official policies,
	either expressed or implied, of gamefreak955.
*/



#include "file_util.h"




int File_Copy(const std::experimental::filesystem::path target, const std::experimental::filesystem::path destination, std::vector<std::string> &extensions)
{
	if (is_directory(target) == false && is_regular_file(target) == false) {
		//std::wcout << "\nINVALID TARGET DIRECTORY " << target.string();
		//std::wcout << "RETURN -1\n\n";
		return -1;
	}
	if (is_directory(destination) == false && is_regular_file(destination) == false) {
		//std::wcout << "\nINVALID DESTINATION DIRECTORY " << destination.string();
		//std::wcout << "RETURN -2\n\n";
		return -2;
	}
	//std::wcout << "ALL STARTUP PARAMETERS ARE VALID";
	if (is_regular_file(target) == true) {
		if (has_extension(target, extensions) == true) {
			std::experimental::filesystem::path final_destination = destination;
			if (is_directory(destination) == true) {
				//final_destination = final_destination.string().append("/").append(target.filename());
				std::experimental::filesystem::system_complete(final_destination);
			}
			std::experimental::filesystem::space_info drive_capacity = std::experimental::filesystem::space(std::experimental::filesystem::path(final_destination.root_path()));
			const unsigned int MINUM_SPACE_PERCENT_RESERVE = 1;
			if (drive_capacity.available < std::experimental::filesystem::file_size(target) || ((MINUM_SPACE_PERCENT_RESERVE * drive_capacity.capacity) / (drive_capacity.available * 100))) {
				//std::wcout << "\n" << "DISK SPACE TOO LOW TO COPY FILE " << target.filename();
				return -3;
			}
			try {
				std::experimental::filesystem::copy_file(target, final_destination, std::experimental::filesystem::copy_options::skip_existing);
			}
			catch (...) {
				//std::wcout << "\nCOPY_FALIURE:"<<"\n";
				//std::wcout << "TARGET=" << target.string()<<"\n";
				//std::wcout << "FINAL_DESTINATION=" << final_destination.string()<<"\n";
				return -3;
			}
			//std::wcout << "Copied: " << target.filename()<<"\n";
			return 1;
		}
		else {
			//std::wcout << "No Relevant Files To Copy"<<"\n";
			return 2;
		}
	}
	//std::wcout << "No Relevant Files To Copy"<<"\n";
	return 3;
}

std::string makequote(const std::string &str)
{
	std::stringstream ss;
	ss << "\"" << str << "\"";
	return ss.str();
}

bool Find_Files(const std::experimental::filesystem::path &dir, std::vector<std::experimental::filesystem::path> &paths)
{
	std::experimental::filesystem::recursive_directory_iterator endit;
	std::experimental::filesystem::recursive_directory_iterator it(dir);
	while (it != endit) {
		auto &apath = it->path();
		if (std::experimental::filesystem::is_directory(apath) && std::experimental::filesystem::is_symlink(apath)) {
			it.pop();
		}
		if (std::experimental::filesystem::is_regular_file(apath)) {
			paths.push_back(*it);
		}
		it++;
	}
	return (paths.size() > 0);
}
std::vector<std::experimental::filesystem::path> Find_Files(const std::experimental::filesystem::path &dir)
{
	std::experimental::filesystem::recursive_directory_iterator endit;
	std::experimental::filesystem::recursive_directory_iterator it(dir);
	std::vector<std::experimental::filesystem::path> paths;
	while (it != endit) {
		auto &apath = it->path();
		if (std::experimental::filesystem::is_directory(apath) && std::experimental::filesystem::is_symlink(apath)) {
			it.pop();
		}
		if (std::experimental::filesystem::is_regular_file(apath)) {
			paths.push_back(*it);
		}
		it++;
	}
	return paths;
}
/*cAsE SeNSItve, include period with the extension name eg .sexy */
bool has_extension(const std::experimental::filesystem::path &fp, const std::vector<std::string> &extensions)
{
	if (std::find(extensions.begin(), extensions.end(), fp.extension()) != extensions.end() == true) {
		return true;
	}
	return false;
}
bool has_extension(const std::experimental::filesystem::path &fp, const std::string &extension)
{
	return (fp.extension() == extension);
}

void press_enter_to_continue()
{
	std::cout << "Press ENTER to continue . . . " << std::flush;
	std::cin.clear();
	std::cin.sync();
	std::cin.ignore(std::numeric_limits <std::streamsize> ::max(), '\n');
}

/*  cAsE SeNSItve, include file extension with name
	eg. COOL_guy.awesome
	Returns:
	SUCCESS = path vector with entries only containing extensions from extensions parameter
	FAILURE = an empty path vector	*/
std::vector<std::experimental::filesystem::path> return_files_with_extentions(const std::vector<std::experimental::filesystem::path> &paths, const std::string &extension)
{
std::vector<std::experimental::filesystem::path> files_with_extensions;
for (std::size_t i = 0; i != paths.size(); i++) {
	if (paths[i].extension() == extension) {
		files_with_extensions.push_back(paths[i]);
	}
}
return files_with_extensions;
}
std::vector<std::experimental::filesystem::path> return_files_with_extentions(const std::vector<std::experimental::filesystem::path> &paths, const std::vector<std::string> &extensions)
{
	//std::transform(data.begin(), data.end(), data.begin(), ::tolower);
	std::vector<std::experimental::filesystem::path> files_with_extensions;
	for (std::size_t i = 0; i != paths.size(); i++) {
		if (std::find(extensions.begin(), extensions.end(), paths[i].extension()) != extensions.end() == true) {
			files_with_extensions.push_back(paths[i]);
		}
	}
	return files_with_extensions;
}




/*  cAsE SeNSItve, include file extension with name
	eg. COOL_guy.awesome
	Returns:
	SUCCESS = path vector with entries only containing names from names parameter
	FAILURE = an empty path vector	*/
std::vector<std::experimental::filesystem::path> get_filenames(const std::vector<std::experimental::filesystem::path> &paths, const std::vector<std::string> &names)
{
	std::vector<std::experimental::filesystem::path> files_with_names;
	for (std::size_t i = 0; i != paths.size(); i++) {
		if (std::find(names.begin(), names.end(), paths[i].filename()) != names.end() == true) {
			files_with_names.push_back(paths[i]);
		}
	}
	return files_with_names;
}
std::vector<std::experimental::filesystem::path> get_filenames(const std::vector<std::experimental::filesystem::path> &paths, const std::string &name)
{
	std::vector<std::experimental::filesystem::path> files_with_names;
	for (std::size_t i = 0; i != paths.size(); i++) {
		if (paths[i].filename() == name) {
			files_with_names.push_back(paths[i]);
		}
	}
	return files_with_names;
}



bool Rename_File(std::experimental::filesystem::path target, std::experimental::filesystem::path destination)
{
	try
	{
		_wrename(target.wstring().c_str(), destination.wstring().c_str());
		//std::experimental::filesystem::rename(target, destination,);
		//std::experimental::filesystem::copy_options::skip_existing
	}
	catch (...)
	{
		//std::wcout <<"\n" << destination.wstring() << " already exists\n";
		return false;
	}
	return true;
}



std::experimental::filesystem::path remove_extention(std::experimental::filesystem::path filename)
{
	if (std::experimental::filesystem::is_regular_file(filename)) {
		std::string remove_ext = filename.filename().string();
		remove_ext = remove_ext.substr(0, remove_ext.size() - 4);
		filename = remove_ext;
	}
	return filename;
}

bool Erase_Temporary_Files(std::experimental::filesystem::path Converter_Filepath)
{
	if (std::experimental::filesystem::is_regular_file(Converter_Filepath) == true)
	{
		Converter_Filepath.remove_filename();
	}
	if (std::experimental::filesystem::is_directory(Converter_Filepath) == true)
	{
		Converter_Filepath.append("4nxci_extracted_xci");
		std::vector<std::experimental::filesystem::path> erase_files = Find_Files(Converter_Filepath);
		std::vector<std::string> extentions;
		extentions.push_back(".nca");
		extentions.push_back(".xml");
		extentions.push_back(".tik");
		extentions.push_back(".cert");
		erase_files = return_files_with_extentions(erase_files, extentions);
		if (erase_files.size() > 0){
			try
			{
				for (std::size_t i = 0; i != erase_files.size(); i++) {
					std::experimental::filesystem::remove(erase_files[i]);
				}
			}
			catch (...)
			{
				return false;
			}
			return true;
		}
		else {
			return false;
		}
	}
	else {
		return false;
	}
}