/*  Copyright @ 2018 gamefreak955

	All rights reserved.
	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:

	1. Redistributions of source code must retain the above copyright notice, this
	list of conditions and the following disclaimer.
	2. Redistributions in binary form must reproduce the above copyright notice,
	this list of conditions and the following disclaimer in the documentation
	and/or other materials provided with the distribution.
	3. Neither the name of this organization nor the names of its contributors may
	be used to endorse or promote products derived from this software without
	specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
	ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
	ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

	The views and conclusions contained in the software and documentation are those
	of the authors and should not be interpreted as representing official policies,
	either expressed or implied, of gamefreak955.
*/




#include "main_functions.h"

using namespace std::experimental::filesystem::v1;


int main(int argc, char *argv[])
{
	std::cout << "XCI to NSP Batch Converter and Renamer Version 0.03\n\n";
	/* Check the number of parameters */
	if (argc < 4) {
		std::cout << "Provide arguments: 'Path to XCI Files' 'Destination for NSP Files' 'Path to 4nxci' \n";
		press_enter_to_continue();
		return -1;
	}
	const std::experimental::filesystem::path XCI_File_DIR = std::experimental::filesystem::system_complete(std::experimental::filesystem::path(argv[1]));
	const std::experimental::filesystem::path NSP_FileDestination = std::experimental::filesystem::system_complete(std::experimental::filesystem::path(argv[2]));
	const std::experimental::filesystem::path Converter_Filepath = std::experimental::filesystem::system_complete(std::experimental::filesystem::path(argv[3]));

	if (Validate_Startup_Parameters(XCI_File_DIR, NSP_FileDestination, Converter_Filepath) < 0) {
		press_enter_to_continue();
		return -4;
	}


	titles Catalogue;
	std::size_t Existing_NSP_Count = 0;
	std::size_t Converted_Count = 0;

	//rename and move existing NSP Files
	Existing_NSP_Count = Rename_NSP_Files(Converter_Filepath, NSP_FileDestination, Catalogue);
	Converted_Count = Convert_XCI_Files(Converter_Filepath, NSP_FileDestination, XCI_File_DIR, Catalogue);
	Erase_Temporary_Files(Converter_Filepath);

	if (Converted_Count > 0 && Existing_NSP_Count > 0 ) {
		std::cout << "\nSUCCESS: " << Converted_Count << " FILES CONVERTED." << Existing_NSP_Count + Converted_Count <<"\nFILES ORGANIZED\n\n";
	}
	else if (Converted_Count > 0) {
		std::cout << "\nSUCCESS: " << Converted_Count << " FILES CONVERTED\n\n";
	}
	else {
		std::cout << Converted_Count + Existing_NSP_Count << " FILES PROCESSED\n\n";
	}
	return 0;
}