/*  Copyright @ 2018 gamefreak955

	All rights reserved.
	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:

	1. Redistributions of source code must retain the above copyright notice, this
	list of conditions and the following disclaimer.
	2. Redistributions in binary form must reproduce the above copyright notice,
	this list of conditions and the following disclaimer in the documentation
	and/or other materials provided with the distribution.
	3. Neither the name of this organization nor the names of its contributors may
	be used to endorse or promote products derived from this software without
	specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
	ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
	ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

	The views and conclusions contained in the software and documentation are those
	of the authors and should not be interpreted as representing official policies,
	either expressed or implied, of gamefreak955.
*/






#include "main_functions.h"


int Validate_Startup_Parameters(const std::experimental::filesystem::path XCI_File_DIR, const std::experimental::filesystem::path NSP_FileDestination, const std::experimental::filesystem::path Converter_Filepath)
{
	if (is_directory(XCI_File_DIR) == false && is_regular_file(XCI_File_DIR) == false) {
		std::cout << "\nINVALID XCI DIRECTORY\n\n" << XCI_File_DIR.string();
		return -1;
	}
	if (is_directory(NSP_FileDestination) == false && is_regular_file(NSP_FileDestination) == false) {
		std::cout << "\nINVALID NSP DESTINATION DIRECTORY\n\n" << NSP_FileDestination.string();
		try
		{
			std::cout << "Attempting to create...\n\n";
			std::experimental::filesystem::create_directories(NSP_FileDestination);
		}
		catch (...)
		{
			std::cout << "ERROR: Could not create directory\n";
			return -2;
		}
		std::cout << "Created valid destination:\n" << NSP_FileDestination << "\n";
		std::cout << "Converted files will be placed there\n\n";
		return 1;
	}
	if (is_regular_file(Converter_Filepath) == false) {
		std::cout << "ERROR: 4nxci.EXE NOT FOUND\n\n";
		return -3;
	}
	return 0;
}



//rename and move any existing nsp files
std::size_t Rename_NSP_Files(const std::experimental::filesystem::path Converter_Filepath, const std::experimental::filesystem::path NSP_FileDestination, titles &Catalogue)
{
	if (is_regular_file(Converter_Filepath) == true && is_directory(NSP_FileDestination) == true ) {
		std::size_t count = 0;
		std::experimental::filesystem::path Converter_Home_Dir = Converter_Filepath;
		Converter_Home_Dir.remove_filename();

		std::vector<std::experimental::filesystem::path> NSP_Files = Find_Files(Converter_Home_Dir);
		NSP_Files = return_files_with_extentions(NSP_Files, ".nsp");

		bool found_nsp_files = (NSP_Files.size() > 0);
		if (found_nsp_files == true) {
			if (Catalogue.data.contents.size() != 0) {
				std::cout << "Renaming NSP Files:\n";
			}
			for (std::size_t i = 0; i != NSP_Files.size(); i++) {
				if (Catalogue.rename_nsp(NSP_Files[i], NSP_FileDestination, Catalogue) > 0) {
					count++;
				}
			}
		}
		return count;
	}
	return 0;
}


//Convert XCI to NSP
std::size_t Convert_XCI_Files(const std::experimental::filesystem::path Converter_Filepath, const std::experimental::filesystem::path NSP_FileDestination, const std::experimental::filesystem::path XCI_File_DIR, titles &Catalogue)
{
	std::size_t count = 0;
	if (is_regular_file(Converter_Filepath) == true && is_directory(XCI_File_DIR) == true) {
		std::vector<std::experimental::filesystem::path> XCI_Files = Find_Files(XCI_File_DIR);
		XCI_Files = return_files_with_extentions(XCI_Files, ".xci");
		bool found_xci_files = (XCI_Files.size() > 0);
		if (found_xci_files == true) {
			std::cout << "Converting XCI files:\n";
			for (std::size_t i = 0; i != XCI_Files.size(); i++) {
				XCI_File XCI;
				Erase_Temporary_Files(Converter_Filepath);
				if (XCI.convert(XCI_Files[i], Converter_Filepath) == 0) {
					count++;
					Rename_NSP_Files(Converter_Filepath, NSP_FileDestination, Catalogue);
				}
				else {
					std::cout << "\nResuming...\n";
				}
			}
		}
		else {
			std::cout << "NO XCI FILES FOUND\n \n";
		}
		Erase_Temporary_Files(Converter_Filepath);
		return count;
	}
	return count;
}