/*  Copyright @ 2018 gamefreak955

	All rights reserved.
	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:

	1. Redistributions of source code must retain the above copyright notice, this
	list of conditions and the following disclaimer.
	2. Redistributions in binary form must reproduce the above copyright notice,
	this list of conditions and the following disclaimer in the documentation
	and/or other materials provided with the distribution.
	3. Neither the name of this organization nor the names of its contributors may
	be used to endorse or promote products derived from this software without
	specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
	ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
	ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

	The views and conclusions contained in the software and documentation are those
	of the authors and should not be interpreted as representing official policies,
	either expressed or implied, of gamefreak955.
*/

#include "titles.h"


titles::titles(void)
{
	get_file_contents();
	extract_data();
}

titles::~titles(void) {}


void titles::get_file_contents(void)
{
	std::experimental::filesystem::path final_destination = "titles.dat";
	if (is_regular_file(final_destination) == true) {
		std::wifstream wfin(final_destination,ios_base::binary);
		wfin.imbue(std::locale(wfin.getloc(), new std::codecvt_utf16<wchar_t, 0x10ffff,std::codecvt_mode(std::little_endian | std::consume_header)>));
		while (wfin.good()) {
			std::wstring read_input = L"";
			std::getline(wfin, read_input);
			data.contents.push_back(read_input);
		}
		wfin.close();
		wfin.clear();
	}
	else {
		std::cout << "\n\n\n\nERROR: TITLES.DAT NOT FOUND\n";
		std::cout << "GAMES WILL NOT BE RENAMED\n\n\n\n";
		//throw (errno);
	}
}

/* invalid names for OS
< (less than)
> (greater than)
: (colon)
g (double quote)
/ (forward slash)
\ (backslash)
| (vertical bar or pipe)
? (question mark)
* (asterisk)
*/
void titles::extract_data(void)
{
	for (std::size_t i = 0; i != data.contents.size(); i++)
	{
		std::size_t start = std::string::npos;
		start = data.contents[i].find(L"| ");
		if (start != std::string::npos) {
			std::size_t end = std::string::npos;
			end = data.contents[i].find(L" || ", start + 2);
			if (end != std::string::npos) {
				std::wstring str = data.contents[i].substr(start + 2, (end - 2 - start));
				str.resize(remove_if(str.begin(), str.end(), [](wchar_t x) {return !iswalnum(x) && !iswspace(x); }) - str.begin());
				std::transform(str.begin(), str.end(), str.begin(), ::towlower);
				data.title.push_back(str);

				start = std::string::npos;
				start = data.contents[i].find(L"| ", end + 2);
				if (start != std::string::npos) {
					end = std::string::npos;
					end = data.contents[i].find(L" || ", start + 2);
					if (end != std::string::npos) {

						std::wstring str = data.contents[i].substr(start + 2, (end - 2 - start));
						str.resize(remove_if(str.begin(), str.end(), [](wchar_t x) {return !iswalnum(x) && !iswspace(x); }) - str.begin());
						data.proper_name.push_back(str);

						start = std::string::npos;
						start = data.contents[i].find(L"| ", end + 2);
						if (start != std::string::npos) {
							end = std::string::npos;
							end = data.contents[i].find(L" || ", start + 2);
							if (end != std::string::npos) {
								std::wstring str = data.contents[i].substr(start + 2, (end - 2 - start));
								str.resize(remove_if(str.begin(), str.end(), [](wchar_t x) {return !iswalnum(x) && !iswspace(x); }) - str.begin());
								data.region.push_back(str);

								start = std::string::npos;
								start = data.contents[i].find(L"| ", end + 2);
								if (start != std::string::npos) {
									end = std::string::npos;
									end = data.contents[i].find(L" || ", start + 2);
									if (end != std::string::npos) {
										data.OS.push_back(data.contents[i].substr(start + 2, (end - 2 - start)));
									}
								}
							}

						}
					}
				}

			}
		}
	}
	return;
}




std::size_t titles::rename_nsp(std::experimental::filesystem::path source_file, std::experimental::filesystem::path destination, titles data)
{

	if (std::experimental::filesystem::is_directory(destination)) {
		std::size_t index = lookup_proper_name(data, source_file);
		if (index  != std::string::npos) {
			wstringstream win;
			win << destination.c_str() << "\\" << data.data.proper_name[index] << " (" << data.data.region[index] << ") NX_" << data.data.OS[index] << ".nsp";
			destination = win.str();
			Rename_File(source_file, destination);
			std::wcout << "\n\nRENAMED: " << source_file.filename() << " to " << data.data.proper_name[index] << " (" << data.data.region[index] << ") NX_" << data.data.OS[index] << ".nsp\n\n";
			return 1;
		}
		else {
			return 0;
		}
	}
	return 0;
}


std::size_t titles::lookup_proper_name(titles data, std::experimental::filesystem::path nsp)
{
	std::wstring filename = nsp.filename().c_str();
	filename = filename.substr(0, filename.size() - 4);

	std::transform(filename.begin(), filename.end(), filename.begin(), ::towlower);
	for (std::size_t i = 0; i != data.data.title.size(); i++) {
		if (data.data.title[i].compare(filename) == 0) {
			return i;
		}
	}
	if (filename.size() == 16 ) {
		std::cout << "\nERROR: TITLE IS NOT IN TITLES.dat: " << nsp.filename() << "\n";
	}
	return std::string::npos;
}